--[[
	Credits
	To Shackleford for LDB-Threat  I had no clue how to write a data source
]]--

local addonName, ns = ...
local addon = LibStub( "AceAddon-3.0" ):NewAddon(addonName, "AceEvent-3.0", "AceTimer-3.0")
local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale(addonName)

local AceCfg = LibStub("AceConfig-3.0")
local AceCfgReg = LibStub("AceConfigRegistry-3.0")
local AceDlg = LibStub("AceConfigDialog-3.0")

local addonversion ="r12"

local DBversion = "1"

local defaultText = "|cffC79C6ELDB|r-LagTolerance"
local defaultIcon = "Interface\\Icons\\Ability_Paladin_ShieldofVengeance"

local Color = {
	red = "ffff0000",
	yellow = "ffffff00",
	orange = "ffffa500",
}

local currentColor = ""
local currentPing = select(3, GetNetStats())
local currentSetting = GetCVar("maxSpellStartRecoveryOffset")

local LDBLagTolerance = LibStub("LibDataBroker-1.1"):NewDataObject(
	addonName, 
	{ 
		icon = defaultIcon, 
		type = "data source",
		text = defaultText,
		OnClick = function(self, button)
			if button == "RightButton" then
				InterfaceOptionsFrame_OpenToCategory("LDB_LagTolerance")
				return
			end
			if button == "LeftButton" then
				if GetCVar("reducedLagTolerance") ~= tostring(1) then
					addon:ChangeLagTolerance()
				end
				addon:UpdateText()
			end
		end
	}
	
)

function LDBLagTolerance:OnTooltipShow()
	self:AddLine(defaultText.." |cff00ff00"..addonversion.."|r")
	self:AddLine("|cffffffff"..L['Tooltip'].."|r")
end

function addon:ChangeLagTolerance()
	currentPing = select(3, GetNetStats())
	if currentPing ~= 0 then
		local newSetting = 0
		if self.db.profile.adjustExact then
			newSetting = math.ceil(math.min(currentPing, 400))
		else
			newSetting = math.ceil(math.min(currentPing, 400) / 10) * 10
		end
		SetCVar("maxSpellStartRecoveryOffset", tostring(newSetting))
		currentSetting = newSetting
	end
end

function addon:RefreshCurrent()
	currentPing = select(3, GetNetStats()) or 0
	if GetCVar("reducedLagTolerance") ~= tostring(1) then
		currentSetting = 400
	else
		currentSetting = GetCVar("maxSpellStartRecoveryOffset")
	end
end

function addon:UpdateText()
	if currentSetting == nil or currentPing == nil then
		self:RefreshCurrent()
	end
	local postfix = ""
	local diff = tonumber(currentSetting - currentPing,10)
	if self.db.profile.showDifference and math.abs(diff) ~= 0 then
		if diff > 150 or diff < -150 then
			currentColor = Color.red
		elseif diff > 80 or diff < -80 then
			currentColor = Color.orange
		elseif diff > 30 or diff < -30 then
			currentColor = Color.yellow
		else 
			currentColor = ""
		end
		
		if diff > 0 then
			if currentColor ~= "" then
				postfix = ' (|c'..currentColor..'+'..diff..'|r)'
			else
				postfix = ' ('..'+'..diff..')'
			end
		else 
			if currentColor ~= "" then
				postfix = ' (|c'..currentColor..diff..'|r)'
			else
				postfix = ' ('..diff..')'
			end
		end
	end
	-- Update data broker values
	LDBLagTolerance.text = currentPing..' ms'..postfix
	LDBLagTolerance.value = currentPing
end

function addon:Update()
	if self.db.profile.autoChange and GetCVar("reducedLagTolerance") == tostring(1) then
		self:ChangeLagTolerance()
	else
		self:RefreshCurrent()
	end
	self:UpdateText()
end

local defaults = {
	profile = {
		autoChange = false,
		showDifference = true,
		adjustExact = false,
		refreshEvery = 30,
		dbVersion = DBversion
	}
}

function addon:OnInitialize()
	-- Database, and initial layout
	self.db = LibStub("AceDB-3.0"):New("LDBLagToleranceDB", defaults, "Default")
	local options = addon:Options()
	AceCfg:RegisterOptionsTable("LDB_LagTolerance", options)
	self.optionsFrame = AceDlg:AddToBlizOptions("LDB_LagTolerance", "LDB_LagTolerance", "Broker")
	options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	self.db.RegisterCallback(self, "OnProfileChanged", "UpdateSettings")
	self.db.RegisterCallback(self, "OnProfileCopied", "UpdateSettings")
	self.db.RegisterCallback(self, "OnProfileReset", "UpdateSettings")
end

function addon:OnEnable()
	self:ScheduleRepeatingTimer("Update", self.db.profile.refreshEvery)
	self:ScheduleTimer("Update", 4)
end

function addon:UpdateSettings()
	self:CancelAllTimers()
	self:ScheduleRepeatingTimer("Update", self.db.profile.refreshEvery)
	self:ScheduleTimer("Update", 4)
end

function addon:Options()
	return {
		type ="group",
		name = defaultText,
		handler = LDBLagTolerance,
		get = function(key) return self.db.profile[key.arg] end,
		set = function(key, value) self.db.profile[key.arg] = value; addon:UpdateSettings() end,
		args = {
			options = {
				name = L["Options"],
				handler = LDBLagTolerance,
				type = "group",
				args = {
					header1 = {
						type = "description",
						name = L["Some useful description"],
						order = 0,
						width = "full",
						cmdHidden = true
					},
					spacer1 = {
						type = "header",
						name = L["Display options"],
						order = 1,
						width = "full",
						cmdHidden = true
					},
					showDifference = {
						type = "toggle",
						name = L["Show the difference between lag tolerance and latency"],
						width = "full",
						order = 2,
						arg = "showDifference"
					},
					spacer2 = {
						type = "header",
						name = L["Lag tolerance adjustment options"],
						order = 3,
						width = "full",
						cmdHidden = true
					},
					autoChange = {
						type = "toggle",
						name = L["Automatically change lag tolerance"],
						width = "full",
						order = 4,
						arg = "autoChange"
					},
					adjustExact = {
						type = "toggle",
						name = L["Adjust exact"],
						width = "full",
						order = 5,
						arg = "adjustExact",
					},
					refreshEvery = {
						name = L["Refresh every X seconds"],
						type = "range",
						min = 5, max = 60, step = 5,
						order = 6,
						arg = "refreshEvery"
					},
				},
			},
		},
	}
end

local brokerOptions = AceCfgReg:GetOptionsTable("Broker", "dialog", "LibDataBroker-1.1")
if (not brokerOptions) then
	brokerOptions = {
		type = "group",
		name = "Broker",
		args = {
		}
	}
	AceCfg:RegisterOptionsTable("Broker", brokerOptions)
	AceDlg:AddToBlizOptions("Broker", "Broker")
end
